package com.topcoder.persistence.relation;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * This abstract class models a relation. A relation consists of a tuple type which
 * indicates the attribute names and types a tuple belonging to the relation can have,
 * a set of tuple constraints indicating the conditions a tuple has to respect to be
 * a part of the tuple and a set of relation constraints indicating global conditions
 * that the entire set of tuples in the relation must respect.
 * 
 * This abstract class provides storage and access functions for the tuple type,
 * tuple constraints and relation constraints. These are common to all relation
 * implementations.
 * 
 * It also provides a set of abstact methods for handling tuples (add, remove, clear,
 * get, find). Neither these methods nor the tuple storage are implemented in this class.
 * They are left to be implemented by the subclasses because it can be done in different
 * ways to achieve different things (better speed, better memory consumption, etc).
 * 
 * Thread safety:
 * 
 * The class is not thread safe because it has state that can mutate.
 * There are lots of modifiers methods and many methods need access to several attributes.
 * That's why synchronizing all methods is the only reasonable solution if thread safety
 * is desired.
 */
public abstract class Relation {
	
	/**
	 * The name of the relation (just for convenince, not used by this component).
	 * 
	 * Should never be null or empty or all spaces.
	 * 
	 * Initialized in the constructor. Used by its setter and getter.
	 */
	private String name = null;
	
	/**
	 * The type of the tuples in this relation.
	 * 
	 * Can be null but no tuples or relation constraints can be added until the type is set.
	 * 
	 * Set in the constructor or setter, used by most of the methods.
	 */
	private TupleType tupleType = null;
	
	/**
	 * The tuple constraint name to tuple constraint mapping.
	 * Maps strings to TupleConstraint instances.
	 * Cannot be null or have null keys and values.
	 * Initialized with HashMap.
	 * Changed by its accessor methods.
	 */	
	private Map tupleConstraints = new HashMap();
	
	/**
	 * The relation constraint name to relation constraint mapping.
	 * Maps strings to RelationConstraint instances.
	 * Cannot be null or have null keys and values.
	 * Initialized with HashMap.
	 * Changed by its accessor methods.
	 */	
	private Map relationConstraints = new HashMap();
	
	/**
	 * Indicates whether relation constraints should be checked.
	 * It has a setter and a getter.
	 * It is used by addTuple, removeTuple, clearTuple, addRelationConstraint, copyRelationConstraints
	 * to see if relation constraints should be enforced.
	 */
	private boolean checkRelationConstraints = true;
	
	/**
	 * Protected constructor to be called by subclass constructors.
	 * 
	 * Implementation details:
	 * 
	 * Copies the parameter to the appropriate attribute.
	 * 
	 * @param name the name of the relation
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	protected Relation(String name) throws IllegalArgumentException {
	}
	
	/**
	 * Protected constructor to be called by subclass constructors.
	 * 
	 * Implementation details:
	 * 
	 * Copies the parameters to the appropriate attributes.
	 * 
	 * @param name the name of the relation
	 * @param tupleType the tuple type of the relation
	 * 
	 * @throws IllegalArgumentException if any argument is null, the string is empty or all spaces
	 */
	protected Relation(String name, TupleType tupleType) throws IllegalArgumentException {
	}
	
	/**
	 * Returns the name of the relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns the name attribute.
	 * 
	 * @return the name of the relation
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * Sets the name of the relation.
	 * 
	 * Implementation details:
	 * 
	 * Sets the name attribute with the parameter value.
	 * 
	 * @param name the name of the relation
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public void setName(String name) throws IllegalArgumentException {
	}

	/**
	 * Returns the tuple type of the relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns the tupleType attribute.
	 * 
	 * @return the tuple type of the relation
	 */
	public TupleType getTupleType() {
		return null;
	}

	/**
	 * Sets the tuple type of the relation (only possible if not already set).
	 * 
	 * Implementation details:
	 * 
	 * Sets the tupleType attribute with the parameter value.
	 * 
	 * @param tupleType the tuple type of the relation
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IllegalStateException if the tuple type has already been set (not null)
	 */	
	public void setTupleType(TupleType tupleType) throws IllegalArgumentException, IllegalStateException {
	}

	// ---
	
	/**
	 * Adds a tuple constraint to the relation.
	 * If a constraint with the same name exists, nothing is done.
	 * The existing tuples are checked to meet the new constraint.
	 * If not met, it is not added and an exception is thrown.
	 * 
	 * Implementation details:
	 * 
	 * - checks the map for the name
	 * - iterates all tuples and checks if they respect the new constraint
	 * - adds the constraint to the map
	 * 
	 * @param name the name of the constraint
	 * @param constraint the constraint to add
	 * 
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if any argument is null, the string is empty or all spaces
	 * @throws EvaluationException in an error occurs while evaluating the constraint for the tuples
	 * @throws ConstraintViolatedException if a tuple violates the constraint
	 */
	public boolean addTupleConstraint(String name, TupleConstraint constraint) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException {
		return false;
	}
	
	/**
	 * Removes a tuple constraint from the relation (by name).
	 * If a constraint with that name doesn't exists, nothing is done.
	 * 
	 * Implementation details:
	 * 
	 * - removes from the map by key
	 * 
	 * @param name the name of the constraint to remove
	 * 
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public boolean removeTupleConstraint(String name) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears the tuple constraints.
	 * 
	 * Implementation details:
	 *
	 * Clears the map.
	 */
	public void clearTupleConstraints() {
	}
	
	/**
	 * Returns the names of the defined tuple constraints.
	 * 
	 * Implementation details:
	 *
	 * Returns the key set of the map.
	 * 
	 * @return the names of the defined tuple constraints
	 */
	public String[] getTupleConstraintNames() {
		return null;
	}
	
	/**
	 * Gets a tuple constraint from the relation (by name).
	 * If a constraint with that name doesn't exists, null is returned
	 * 
	 * Implementation details:
	 * 
	 * - gets from the map
	 * 
	 * @param name the name of the constraint to get
	 * 
	 * @return the constraint or null if no constraint with the given name exists
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public TupleConstraint getTupleConstraint(String name) throws IllegalArgumentException {
		return null;
	}
	
	/**
	 * Copies the tuple constraints from a given relation.
	 * This is useful for the relations obtained from operations which have all constraints stripped.
	 * Before copying, all tuples are checked to see if they meet all constraints.
	 * 
	 * Implementation details:
	 * 
	 * - iterate each tuple and see if it meets all tuple constraints from the given relation
	 * - if yes, clear map and copy the constraints
	 * 
	 * @param relation the relation to copy constraints from
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException in an error occurs while evaluating the constraint for the tuples
	 * @throws ConstraintViolatedException if a tuple violates the constraint
	 */
	public void copyTupleConstraints(Relation relation) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException {
	}
	
	// ---

	/**
	 * Adds a relation constraint to the relation.
	 * If a constraint with the same name exists, nothing is done.
	 * The relation is checked to meet the new constraint.
	 * If not met, it is not added and an exception is thrown.
	 * 
	 * Implementation details:
	 * 
	 * - checks the map for the name
	 * - calls the constraint to see if currently met (if checkRelationConstraints == true)
	 * - if yes, adds the constraint to the map
	 * 
	 * @param name the name of the constraint
	 * @param constraint the constraint to add
	 * 
	 * @return true if added, false if not
	 * 
	 * @throws IllegalArgumentException if any argument is null, the string is empty or all spaces
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates the constraint
	 * @throws IllegalStateException if the tuple type has not been set yet
	 */
	public boolean addRelationConstraint(String name, RelationConstraint constraint) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException, IllegalStateException {
		return false;
	}
	
	/**
	 * Removes a relation constraint from the relation (by name).
	 * If a constraint with that name doesn't exists, nothing is done.
	 * 
	 * Implementation details:
	 * 
	 * - removes from the map by key
	 * 
	 * @param name the name of the constraint to remove
	 * 
	 * @return true if removed, false if not
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public boolean removeRelationConstraint(String name) throws IllegalArgumentException {
		return false;
	}
	
	/**
	 * Clears the relation constraints.
	 * 
	 * Implementation details:
	 *
	 * Clears the map.
	 */
	public void clearRelationConstraints() {
	}
	
	/**
	 * Returns the names of the defined relation constraints.
	 * 
	 * Implementation details:
	 *
	 * Returns the key set of the map.
	 * 
	 * @return the names of the defined relation constraints
	 */
	public String[] getRelationConstraintNames() {
		return null;
	}
	
	/**
	 * Gets a relation constraint from the relation (by name).
	 * If a constraint with that name doesn't exists, null is returned
	 * 
	 * Implementation details:
	 * 
	 * - gets from the map
	 * 
	 * @param name the name of the constraint to get
	 * 
	 * @return the constraint or null if no constraint with the given name exists
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public TupleConstraint getRelationConstraint(String name) throws IllegalArgumentException {
		return null;
	}
	
	/**
	 * Set the relation constraint check flag.
	 * If same value as the current value, nothing is done.
	 * 
	 * Implementation details:
	 * 
	 * - if new value is true and old value is false, iterate each relation constraint
	 *   and check if the relation meets it
	 * - the flag is set to the new value only if all constraints are met
	 *   or if changing from true to false
	 * 
	 * @param checkRelationConstraints the new value of the flag
	 * 
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates any constraint
	 */
	public void setCheckRelationConstraints(boolean checkRelationConstraints) throws EvaluationException, ConstraintViolatedException {
	}
	
	/**
	 * Get the relation constraint check flag.
	 * 
	 * Implementation details:
	 * 
	 * Returns the checkRelationConstraints field.
	 * 
	 * @return the relation constraint check flag
	 */
	public boolean isCheckRelationConstraints() {
		return false;
	}

	/**
	 * Copies the relation constraints from a given relation.
	 * This is useful for the relations obtained from operations which have all constraints stripped.
	 * Before copying, the relation is checked to see if it meets all constraints.
	 * 
	 * Implementation details:
	 * 
	 * - iterate each given relation constraint and check if constraint is met for this relation
	 *    (if checkRelationConstraints == true)
	 * - if yes, clear map and copy the constraints
	 * 
	 * @param relation the relation to copy constraints from
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 * @throws IllegalStateException if the tuple type has not been set yet
	 */
	public void copyRelationConstraints(Relation relation) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException, IllegalStateException {
	}
	
	// ---

	/**
	 * Adds a tuple to the relation.
	 * Normally if the tuple already exists, it is ignored.
	 * 
	 * However, for some implementations it might be too expensive to implement duplicate detection
	 * so they are allowed not to enforce it. However, the user is expected in that case to ensure
	 * duplicates aren't added. Memory space efficient implementations could need O(n) to detect
	 * duplicates so they are possible not to implement duplicate checking.
	 * 
	 * All tuple constraints are evaluated for the tuple to see if it meets them.
	 * If checkRelationConstraints == true, all relation constraints are checked too.
	 * If any constraint fails, an exception is thrown and the tuple is not added.
	 * 
	 * @param tuple the tuple to add
	 * 
	 * @return true if added, false if not (duplicate)
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 * @throws IllegalStateException if the tuple type has not been set yet
	 */
	public abstract boolean addTuple(Tuple tuple) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException, IllegalStateException;

	/**
	 * Removes a tuple from the relation.
	 * If the tuple doesn't exists, nothing is done.
	 * 
	 * If checkRelationConstraints == true, all relation constraints are checked.
	 * If any constraint fails, an exception is thrown and the tuple is not removed.
	 * 
	 * @param tuple the tuple to remove
	 * 
	 * @return true if removed, false if not (not found)
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 */
	public abstract boolean removeTuple(Tuple tuple) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException;
	
	/**
	 * Removes all tuples from the relation.
	 * 
	 * If checkRelationConstraints == true, all relation constraints are checked.
	 * If any constraint fails, an exception is thrown and the clear is not performed.
	 * 
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 */
	public abstract void clearTuples() throws EvaluationException, ConstraintViolatedException;
	
    /**
     * Returns the tuple count.
     * 
     * @return the tuple count.
     */
	public abstract int getTupleCount();
	
	/**
	 * Returns all tuples from the relation.
	 * 
	 * @return all tuples from the relation
	 */
	public abstract Tuple[] getAllTuples();
	
	/**
	 * Returns an iterator for the tuples from the relation.
	 * This is meant to be an efficient way to check the tuples.
	 * The array returning version is not that efficient because an array has to be created. 
	 * 
	 * @return the tuple iterator
	 */	
	public abstract Iterator getTuplesIterator();
	
    /**
     * Basic find method that finds tuples by an attribute value.
     * For anything that is more complex, probably the restrict operation is a better choice.
     * 
     * @param attribute the attribute to find by
     * @param value the value of the attribute
     * 
     * @return the found tuples
     * 
     * @throws IllegalArgumentException if any argument is null or if the string is empty or all spaces
     */	
	public abstract Tuple[] findTuples(String attribute, Object value) throws IllegalArgumentException;
	
    /**
     * Basic find method that finds tuples by some attribute values.
     * For anything that is more complex, probably the restrict operation is a better choice.
     * 
     * @param attributes the attributes to find by
     * @param values the values of the attributes
     * 
     * @return the found tuples
     * 
     * @throws IllegalArgumentException if any argument is null, if the arrays have different size or
     * if they have null elements, empty or all spaces strings
     */	
	public abstract Tuple[] findTuples(String attributes[], Object values[]) throws IllegalArgumentException;
	
	// ---
	
	/**
	 * Notifier method called by Tuple when a value for a field changes.
	 * This gives the chance to enforce the tuple and relation constraints.
	 * 
	 * Implementation details:
	 * 
	 * - iterates all tuple constraints and verifies that the tuple meets them
	 * - iterates all relation constraints (only if checkRelationConstraints == true)
	 * - if StatefulRelationConstraint, call tupleChanged to see if the constraint is still met
	 * - if just a simple RelationConstraint, call validate to see if the constraint is met 
	 * 
	 * @param tuple the tuple (changed)
	 * @param attribute the attribute that has changed
	 * @param oldValue the old value of the attribute
	 * 
	 * @throws IllegalArgumentException if any argument is null or if the string is empty or all spaces
	 * @throws EvaluationException in an error occurs while evaluating the constraint
	 * @throws ConstraintViolatedException if the relation violates a constraint
	 */
	void tupleChanged(Tuple tuple, String attribute, Object oldValue) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException {	
	}

}
