package com.topcoder.persistence.relation;


/**
 * This interface is a special extension of the RelationConstraint interface
 * that allows some implementations to be more efficient by keeping state.
 * 
 * For example, a unique attribute enforcing implementation may keep a HashSet
 * of all attribute values, as tuples are added and removed. Without it, the
 * implementation would have to iterate all records each time a tuple is added
 * or removed.
 * 
 * The interface defines three notification and validation methods. These
 * methods are called when tuples are added, removed or changed. 
 * 
 * Thread safety:
 * 
 * The implementations of this class are expected to have no state or be
 * immutable so they should be thread safe.
 * 
 * @author TCSDESIGNER
 */
public interface StatefulRelationConstraint extends RelationConstraint {

	/**
	 * Called to notify that a tuple has been added.
	 * In response a boolean value is returned to indicate whether the
	 * constrain is met if the addition is accepted. 
	 * 
	 * @param tuple the added tuple
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean tupleAdded(Tuple tuple) throws IllegalArgumentException, EvaluationException;

	/**
	 * Called to notify that a tuple has been remove.
	 * In response a boolean value is returned to indicate whether the
	 * constrain is met if the removal is accepted. 
	 * 
	 * @param tuple the removed tuple
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
    public boolean tupleRemoved(Tuple tuple) throws IllegalArgumentException, EvaluationException;

    /**
	 * Called to notify that a tuple has been changed.
	 * In response a boolean value is returned to indicate whether the
	 * constrain is met if the change is accepted. 
	 * 
	 * @param tuple the changed tuple
	 * @param attribute the changed attribute
	 * @param oldValue the old value
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if any argument is null or if the string is empty or all spaces
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean tupleChanged(Tuple tuple, String attribute, Object oldValue) throws IllegalArgumentException, EvaluationException;

}
