package com.topcoder.persistence.relation;

import java.util.HashMap;
import java.util.Map;

/**
 * This class represents a tuple.
 * A tuple is a set of attribute names and their values.
 * 
 * In addition to storing the attribute names and their values,
 * this class also stores the relation the tuple belongs to.
 * This is used to notify the relation of any value changes
 * (so constraints can be enforced).
 * 
 * It implements equals to ease the testing of whether two
 * tuples are the same (along hashCode, widely used in this design since tuples are
 * usually stored in HashSets). 
 * 
 * Thread safety:
 * 
 * This class is mutable (setValue changes values, setRelation changes relation).
 * This means it is not thread safe. Even if synchonized methods or blocks are used to access
 * these fields, two threads should not work with the same Tuple simultaneously because
 * if one changes the tuple, the other might not expect it and have problems. 
 * 
 * @author TCSDESIGNER
 */
public class Tuple implements Cloneable {
	
	/**
	 * The attribute name to attribute value mapping.
	 * Maps strings to Objects.
	 * Cannot be null or have null keys and values.
	 * Initialized with HashMap.
	 * Filled in the constructor and used in all the other methods.
	 * Modified by setValue.
	 */
	private Map values = new HashMap();
	
	/**
	 * The relation the tuple belongs to.
	 * Can be null (not in a relation yet).
	 * The setter and getter use it.
	 * setValue uses it to notify the relation of a value change.
	 */
	private Relation relation = null;
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * Puts all attribute name - attribute value pairs in the values map.
	 *  
	 * @param attributes the attribute names
	 * @param values the attribute values
	 * 
	 * @throws IllegalArgumentException if the arguments are null, have null element,
	 * have different sizes or if the attribute Strings are empty or all spaces
	 */
	public Tuple(String[] attributes, Object[] values) throws IllegalArgumentException {
	}
	
	/**
	 * Returns the attribute names.
	 * 
	 * Implementation details:
	 * 
	 * Build the array from the key set of the types map.
	 * 
	 * @return the attribute names
	 */
	public String[] getAttributes() {
		return null;
	}

	/**
	 * Returns the attribute value of an attribute.
	 * 
	 * Implementation details:
	 * 
	 * Gets the value from the map.
	 * 
	 * @param attribute the attribute name
	 * 
	 * @return the value of the attribute (null if no such attribute exists)
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public Object getValue(String attribute) throws IllegalArgumentException {
		return null;
	}
	
	/**
	 * Sets the value for a given attribute.
	 * 
	 * Implementation details:
	 * 
	 * If relation is not null, it calls Relation.tupleChanged no notify the relationship of the change.
	 * Then it actually sets the value in the map
	 * 
	 * @param attribute the attribute to set
	 * @param value the value to set
	 * 
	 * @throws IllegalArgumentException if the string is null, empty or all spaces or if value is null
	 * @throws EvaluationException if an error occured while evaluating a constraint
	 * @throws ConstraintViolatedException if a constraint is violated by the new value
	 */
	public void setValue(String attribute, Object value) throws IllegalArgumentException, EvaluationException, ConstraintViolatedException {
		// notify relation
	}
	
	/**
	 * Sets the relation the tuple belongs to (internal method for the component).
	 * Called by Relation implementations when a tuple is added / removed.
	 * 
	 * Implementation details:
	 * 
	 * Sets the relation attribute.
	 * 
	 * @param relation the relation the tuple belongs to
	 */
	void setRelation(Relation relation) {
	}
	
	/**
	 * Returns the relation the tuple belongs to.
	 * 
	 * Implementation details:
	 * 
	 * Returns the relation attribute.
	 * 
	 * @return the relation the tuple belongs to
	 */
	public Relation getRelation() {
		return null;
	}
	
	/**
	 * Returns whether this tuple is equal with the given object
	 * 
	 * Implemenation details:
	 * 
	 * - object must be a Tuple to return true
	 * - the values map of the two objects must be equal to return true
	 * 
	 * @param object the object to compare to
	 * 
	 * @return true is equal, false is not
	 */
	public boolean equals(Object object) {
		return false;
	}
	
	/**
	 * Returns the hashcode of the object.
	 * 
	 * Implementation details:
	 * 
	 * Returns values.hashCode().
	 * 
	 * @return the hashcode
	 */
	public int hashCode() {
		return 0;
	}
	
	/**
	 * Clones this object and returns the copy.
	 * The copy will have null relation (meaning no relation contains it yet).
	 * 
	 * Implementation details:
	 * 
	 * Creates a tuple with the same attribute names and values.
	 * 
	 * @return the cloned copy of the object 
	 */
	public Object clone() {
		return null;
	}

}
