package com.topcoder.persistence.relation.constraints;

import com.topcoder.persistence.relation.EvaluationException;
import com.topcoder.persistence.relation.Relation;
import com.topcoder.persistence.relation.RelationConstraint;

/**
 * This class implements a relation constraint that allows specifying
 * mathematical conditions for the aggregated total for a field.
 * 
 * The aggregated total can be a sum, an average, minimum or maximum.
 * The condition for the total is expressed as a Math Expression
 * expression (that returns a boolean result). The expression used
 * "result" as placeholder for the result of the aggregation
 * (for example SUM and "result = 100" means the SUM of all tuples
 * on the given attribute must be 100).
 * 
 * The attribute being aggregated must be of numeric type.
 * 
 * Thread safety:
 * 
 * Immutable class so there are no thread safety issues. 
 * 
 * @author TCSDESIGNER
 */
public class AggregateConstraint implements RelationConstraint {
	
	/**
	 * Constant for SUM attribute aggregation.
	 */
	public static final int SUM = 1;
	
	/**
	 * Constant for AVERAGE attribute aggregation.
	 */
	public static final int AVG = 2;
	
	/**
	 * Constant for MINIMUM attribute aggregation.
	 */
	public static final int MIN = 3;

	/**
	 * Constant for MAXIMUM attribute aggregation.
	 */
	public static final int MAX = 4;

	/**
	 * Constant for COUNT attribute aggregation.
	 */
	public static final int COUNT = 5;

	/**
	 * The attribute the aggregation is performed on.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */
	private String attribute = null;

	/**
	 * The aggregation type (see the constants).
	 * Cannot be anything but the defined constants
	 * Set in the constructor, used in validate.
	 */
    private int aggregationType = 0;
    
	/**
	 * The expression that is used to test the result of the aggregation.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */    
	private String expression = null;

	/**
	 * Constructor. Copies the params to the attributes.
	 * 
	 * @param attribute the attribute to aggregate
	 * @param aggregationType the aggregation type
	 * @param expression the expression to test the result of the aggregation
	 * 
	 * @throws IllegalArgumentException if any argument is null, empty string or all spaces
	 * or if aggregationType is not one of the constants defined in this class
	 */
	public AggregateConstraint(String attribute, int aggregationType, String expression) throws IllegalArgumentException {
	}
	
	/**
	 * Getter for the aggregation attribute (attribute field).
	 * 
	 * @return the aggregation attribute
	 */
	public String getAttribute() {
		return null;
	}

	/**
	 * Getter for the aggregation type (aggregationType field).
	 * 
	 * @return the aggregation type
	 */
	public int getAggregationType() {
		return 0;
	}

	/**
	 * Getter for the test expression (expression field).
	 *  
	 * @return the expression
	 */
	public String getExpression() {
		return null;
	}

	/**
	 * Checks whether a relation respects the constraint.
	 * It involves iterating each relation tuple, aggregating the required
	 * attribute and evaluating the test expression.
	 * 
	 * Implementation details:
	 * 
	 * - for COUNT the result is known immediately without iterating tuples
	 * - iterate each tuple and keep the minimum, maximum, sum
	 * - depending on the aggregation type, assign result
	 * - MathEval me = new MathEval()
	 * - me.define("f(result)->" + expression)
	 * - me.evaluate("f(" + result + ")")) should return a boolean result
	 * 
	 * @param relation the relation to check
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Relation relation)  throws IllegalArgumentException, EvaluationException {
		return false;
	}
	
}
