package com.topcoder.persistence.relation.constraints;

import com.topcoder.persistence.relation.EvaluationException;
import com.topcoder.persistence.relation.Tuple;
import com.topcoder.persistence.relation.TupleConstraint;

/**
 * This class represents a tuple constraint that specifies that an
 * attribute must be within a given range.
 * 
 * The range can be inclusive or not at each end.
 * 
 * This constraint uses Comparable.compareTo for comparison.
 * That means that the types used for the compared fields must
 * implement Comparable.
 * 
 * Thread safety:
 * 
 * Immutable class so there are no thread safety issues. 
 * 
 * @author TCSDESIGNER
 */
public class BetweenConstraint implements TupleConstraint {

	/**
	 * The attribute the check is performed on.
	 * Cannot be null, empty or all spaces.
	 * Set in the constructor, used in validate.
	 */
	private String attribute = null;
	
	/**
	 * The left (from) value the attribute is compared to.
	 * Cannot be null.
	 * Set in the constructor, used in validate.
	 */
	private Object fromValue = null;

	/**
	 * The right (to) value the attribute is compared to.
	 * Cannot be null.
	 * Set in the constructor, used in validate.
	 */
	private Object toValue = null;
	
	/**
	 * Inclusive left (true) or exclusive left (false) indicator.
	 * Set in the constructor. Defaults to true.
	 */
	private boolean inclusiveLeft = true;

	/**
	 * Inclusive right (true) or exclusive right (false) indicator.
	 * Set in the constructor. Defaults to true.
	 */
    private boolean inclusiveRight = true;
	
	/**
	 * Constructor. Copies the params to the attributes. 
	 * 
	 * @param attribute the attribute to compare
	 * @param fromValue the left margin of the interval
	 * @param toValue the right margin of the interval
	 * 
	 * @throws IllegalArgumentException if any argument is null or if attribute is empty or all spaces
	 */    
    public BetweenConstraint(String attribute, Object fromValue, Object toValue) throws IllegalArgumentException {
	}

	/**
	 * Constructor. Copies the params to the attributes. 
	 * 
	 * @param attribute the attribute to compare
	 * @param fromValue the left margin of the interval
	 * @param toValue the right margin of the interval
	 * @param inclusiveLeft if left margin should be inclusive
	 * @param inclusiveRight if right margin should be inclusive
	 * 
	 * @throws IllegalArgumentException if any argument is null or if attribute is empty or all spaces
	 */    
	public BetweenConstraint(String attribute, Object fromValue, Object toValue, boolean inclusiveLeft, boolean inclusiveRight) throws IllegalArgumentException {
	}

	/**
	 * Getter for the attribute (attribute field).
	 * 
	 * @return the attribute
	 */
	public String getAttribute() {
		return null;
	}

	/**
	 * Getter for the compare from value (fromValue field).
	 * 
	 * @return the compare from value
	 */
	public Object getFromValue() {
		return null;
	}

	/**
	 * Getter for the compare to value (toValue field).
	 * 
	 * @return the compare to value
	 */
	public Object getToValue() {
		return null;
	}

	/**
	 * Returns whether between interval is inclusive on the left side (the value of the inclusiveLeft attribute).
	 * 
	 * @return if inclusive left
	 */
	public boolean isInclusiveLeft() {
		return inclusiveLeft;
	}

	/**
	 * Returns whether between interval is inclusive on the right side (the value of the inclusiveRight attribute).
	 * 
	 * @return if inclusive right
	 */
	public boolean isInclusiveRight() {
		return inclusiveRight;
	}
	
	/**
	 * Checks whether the tuple respects the constraint.
	 * 
	 * Implementation details:
	 * 
	 * - attrValue = tuple.getValue(attribute)
	 * - inclusiveLeft true, attrValue.compareTo(fromValue) >= 0, otherwise return false 
	 * - inclusiveLeft false, attrValue.compareTo(fromValue) > 0, otherwise return false
	 * - inclusiveRight true, attrValue.compareTo(toValue) <= 0, otherwise return false
	 * - inclusiveRight false, attrValue.compareTo(toValue) < 0, otherwise return false
	 * - return true
	 * 
	 * @param tuple the tuple to check
	 * 
	 * @return true if constraint is met, false otherwise
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws EvaluationException thrown if the evaluation cannot be done because of an
	 * implementation specific error 
	 */
	public boolean validate(Tuple tuple) throws IllegalArgumentException, EvaluationException {
		return false;
	}
	
}
