package com.topcoder.persistence.relation.operations;

import com.topcoder.persistence.relation.BinaryOperation;
import com.topcoder.persistence.relation.IncompatibleRelationsException;
import com.topcoder.persistence.relation.Relation;

/**
 * Implementation of a binary operation for the difference of two relations.
 * The result of this operation is a new relation that contains all the tuples
 * from the left relation that are not present in the right relation.
 * 
 * This operation applies only to relations with the same tuple type
 * (same attribute names, same attribute types). See TupleType.equals.
 *
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * Thread safety:
 * 
 * This class has no state so it is thread safe.
 * 
 * @author TCSDESIGNER
 */
public class DifferenceOperation implements BinaryOperation {

	/**
	 * Default constructor that doesn't do anything.
	 */
	public DifferenceOperation() {
	}

	/**
	 * Computes the difference of the two relations.
	 * The result relation will contain all tuples from the left relation that
	 * are not present in the right relation.
	 * 
	 * Implementation details:
	 * 
	 * The difference algorithm is simply putting all tuples from the left
	 * relation in a HashSet and then iterating all tuples from the right
	 * relation and removing them from the set (in case they exist there).
	 * Each HashSet insertion or removal is O(1) so the complexity is O(m+n)
	 * where m and n are the sizes of the two relations.
	 * 
	 * - check whether leftRelation.getTupleType().equals(rightRelation.getTupleType())
	 * - the tuple type of resultRelation is copied from any of the two relations
	 * - creates a new HashSet
	 * - calls leftRelation.getTuplesIterator()
	 * - iterates all tuples using the iterator and adds them to the set
	 * - calls rightRelation.getTuplesIterator()
	 * - iterates all tuples using the iterator and removes them from the set
	 * - adds all tuples from the set to the result relation
	 * 
	 * @param leftRelation the left relation
	 * @param rightRelation the right relation
	 * @param resultRelation the result relation (must be empty and typeless)
	 * 
	 * @throws IllegalArgumentException if any argument is null or the result relation is not empty and typeless
	 * @throws IncompatibleRelationsException if the tuple types of the two relations are not equal
	 */
	public void compute(Relation leftRelation, Relation rightRelation, Relation resultRelation) throws IllegalArgumentException, IncompatibleRelationsException {
	}

}
