package com.topcoder.persistence.relation.operations;

import com.topcoder.persistence.relation.BinaryOperation;
import com.topcoder.persistence.relation.Tuple;

/**
 * Base class for join implementations.
 * This class stores common attributes that the joins use such as
 * attribute aliases and missing value tuple for outer joins.
 * 
 * Aliases are used when two relations have common attribute names.
 * Joining them would result in name conflict so aliasing allows
 * renaming fields in order to avoid that.
 * 
 * Outer joins are supported by keeping a special tuple, the missingValues
 * tuple. When a tuple from the left relation doesn't have a match in the
 * right relation, the left tuple is combined with the special missingValues
 * tuple. This approach is chosen to respect the requirement saying nulls
 * should not be used (the most natural value to be used for missing values).
 *
 * The result relation is given by the user (must be empty and typeless).
 * This allows the user to select the desired relation implementation
 * (optimized for speed, for memory, etc).
 * 
 * Thread safety:
 * 
 * This class is immutable so it is thread safe.
 * 
 * @author TCSDESIGNER
 */
public abstract class JoinOperation implements BinaryOperation {
	
	/**
	 * The aliased attributes for the left relation.  
	 * Can be null but cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor and used in compute.
	 * leftAliased and leftAliases must both be null or both have
	 * the same size.
	 */
	private String[] leftAliased = null;
	
	/**
	 * The aliases for the attributes in leftAliased.  
	 * Can be null but cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor and used in compute.
	 * leftAliased and leftAliases must both be null or both have
	 * the same size.
	 */
	private String[] leftAliases = null;

	/**
	 * The aliased attributes for the right relation.  
	 * Can be null but cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor and used in compute.
	 * rightAliased and rightAliases must both be null or both have
	 * the same size.
	 */
	private String[] rightAliased = null;

	/**
	 * The aliases for the attributes in rightAliased.  
	 * Can be null but cannot contain nulls, empty or all spaces strings.
	 * Set in the constructor and used in compute.
	 * rightAliased and rightAliases must both be null or both have
	 * the same size.
	 */
	private String[] rightAliases = null;
	
	/**
	 * The tuple that should be used for missing values in outer joins.
	 * When a left tuple has no match, it is combined with this tuple.
	 * Can be null (meaning a normal join is performed).
	 * Non-null means an outer join is to be performed.
	 * Set in the constructors and used by compute. 
	 */
	private Tuple missingValues = null;

	/**
	 * Protected constructor that doesn't specify anything (aliases, missing values).
	 * This will construct a normal (inner) join.
	 */
	protected JoinOperation() {
	}

	/**
	 * Protected constructor specifying the aliases.
	 * This will construct a normal (inner) join.
	 * 
	 * Implementation details:
	 * 
	 * Copies the parameters to the corresponding attributes (defensive array copies are made).
	 * 
     * @param leftAliased the attributes from the left relation that are aliased 
     * @param leftAliases the aliases for the aliased attributes from the left table
     * @param rightAliased the attributes from the right relation that are aliased
     * @param rightAliases the aliases for the aliased attributes from the right table
     * 
     * @throws IllegalArgumentException if any argument is null, has null elements or 
	 * empty or all spaces strings or if the sizes of the pair arrays are different
     */
	public JoinOperation(String[] leftAliased, String[] leftAliases, String[] rightAliased, String[] rightAliases) throws IllegalArgumentException {
	}

	/**
	 * Protected constructor that will construct an outer join (the tuple with the missing values is given).
	 * 
	 * Implementation details:
	 * 
	 * Copies the parameter to the corresponding attribute.
	 * 
     * @param missingValues the tuple specifying the missing values to be used for outer joins
     * 
     * @throws IllegalArgumentException if the argument is null
     */
	public JoinOperation(Tuple missingValues) throws IllegalArgumentException {
	}

	/**
	 * Protected constructor specifying the aliases.
	 * This will construct an outer join (a tuple with the missing values is given too).
	 * 
	 * Implementation details:
	 * 
	 * Copies the parameters to the corresponding attributes (defensive array copies are made).
	 * 
     * @param leftAliased the attributes from the left relation that are aliased 
     * @param leftAliases the aliases for the aliased attributes from the left table
     * @param rightAliased the attributes from the right relation that are aliased
     * @param rightAliases the aliases for the aliased attributes from the right table
     * @param missingValues the tuple specifying the missing values to be used for outer joins
     * 
     * @throws IllegalArgumentException if any argument is null, has null elements or 
	 * empty or all spaces strings or if the sizes of the pair arrays are different
     */
	public JoinOperation(String[] leftAliased, String[] leftAliases, String[] rightAliased, String[] rightAliases, Tuple missingValues) throws IllegalArgumentException {
	}

	/**
	 * Returns the aliased attributes for the left relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns the leftAliased attribute (defensive copy).
	 * 
	 * @return the aliased attributes for the left relation
	 */
	public String[] getLeftAliased() {
		return leftAliased;
	}

	/**
	 * Returns the aliases for the aliased attributes in the left relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns the leftAliases attribute (defensive copy).
	 * 
	 * @return the aliases for the aliased attributes in the left relation
	 */
	public String[] getLeftAliases() {
		return leftAliases;
	}

	/**
	 * Returns the aliased attributes for the right relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns the rightAliased attribute (defensive copy).
	 * 
	 * @return the aliased attributes for the right relation
	 */
	public String[] getRightAliased() {
		return rightAliased;
	}

	/**
	 * Returns the aliases for the aliased attributes in the right relation.
	 * 
	 * Implementation details:
	 * 
	 * Returns the rightAliases attribute (defensive copy).
	 * 
	 * @return the aliases for the aliased attributes in the right relation
	 */
	public String[] getRightAliases() {
		return rightAliases;
	}
	
	/**
	 * Returns the tuple that should be used for missing values in outer joins.
	 * 
	 * Implementation details:
	 * 
	 * Returns the missingValues attribute.
	 * 
	 * @return the tuple that should be used for missing values in outer joins
	 */
	public Tuple getMissingValues() {
		return missingValues;
	}
	
}
